﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.IO;
using System.Data;
using VoiceElements.Common;

namespace VoiceApp
{
    /// <summary>
    /// This class is responsible for assigning outbound threads.
    /// </summary>
    public class DialManager
    {
        public DialManager()
        {
            AliveEvent = new ManualResetEvent(false);
        }

        private string m_WorkingFolder = IvrApplication.WorkingFolder;

        private Log m_Log = IvrApplication.Log;

        /// <summary>
        /// This is used to sync our threads.
        /// </summary>
        private object m_SyncVar = new object();

        /// <summary>
        /// This is a queue of all the calls that will be going out.
        /// </summary>
        private Queue<AppointmentReminder> m_Queue = new Queue<AppointmentReminder>();

        /// <summary>
        /// Set the signal on this to stop processing.
        /// </summary>
        public ManualResetEvent AliveEvent
        {
            get;
            set;
        }

        /// <summary>
        /// This is used to keep track of the number of calls that we have active.
        /// </summary>
        private int m_ActiveCallCount = 0;

        /// <summary>
        /// Thread safe way to increment a call counter
        /// </summary>
        public void IncrementCallCounters()
        {
            Interlocked.Increment(ref m_ActiveCallCount);
        }

        /// <summary>
        /// Thread safe way to decrement a call counter
        /// </summary>
        public void DecrementCallCounters()
        {
            Interlocked.Decrement(ref m_ActiveCallCount);
        }

        /// <summary>
        /// This thread is responsibile for reading the queue and starting up other threads to place calls
        /// </summary>
        public void PlaceCalls()
        {
            m_Log.Write("Started PlaceCalls thread");
            while (true)
            {
                AppointmentReminder reminder = null;
                // When true this means that we have ports available to dial out on.
                if (m_ActiveCallCount < Properties.Settings.Default.MaximumChannels)
                {
                    // This means that we are OK to dial out on.
                    lock (m_SyncVar)
                    {
                        // Make sure we have values to read
                        if (m_Queue.Count > 0)
                        {
                            reminder = m_Queue.Dequeue();
                        }
                    }

                    // Check to make sure we have a value
                    if (reminder != null)
                    {
                        // Create a new instance of the dial class
                        OutboundDial dial = new OutboundDial(IvrApplication.s_TelephonyServer, this);

                        // Create a new thread that the dial class will run on
                        System.Threading.Thread oThread = new Thread(dial.RunScript);

                        // Set the name of the thread. This makes it easy to look at the logs
                        oThread.Name = reminder.UniqueId;

                        // Set the call data value which contains the logic to use for the call
                        dial.CallData = reminder;

                        // Increment the call count
                        IncrementCallCounters();

                        // Start the thread
                        oThread.Start();
                    }
                    else
                    {
                        // If there's nothing left, we want to sleep for a period of time. This prevents the thread from reaching max core usage
                        bool isSignalled = AliveEvent.WaitOne(500);

                        // If is signalled is true, that means the application has stopped procesisng.
                        if (isSignalled)
                        {
                            break;
                        }
                    }
                }
                else
                {
                    // This waits less time to see if a channel resource is available to use
                    bool isSignalled = AliveEvent.WaitOne(100);

                    // If the reset event is signalled then we go ahead and break.
                    if (isSignalled)
                    {
                        break;
                    }
                }
            }

            m_Log.Write("Ended PlaceCalls thread");
        }

        /// <summary>
        /// This thread is responsible for checking to see if new files exist and queuing the calls.
        /// </summary>
        public void ReadFiles()
        {
            m_Log.Write("Started ReadFiles Thread");
            while (true)
            {
                // Get all of the files in the current directory. These contain lists of phone numbers to dial
                string[] filenames = Directory.GetFiles(m_WorkingFolder + @"\CSV\");

                // Loop through all of the filenames
                // It would be easy to grab values from databases instead
                foreach (string file in filenames)
                {
                    FileInfo fi = new FileInfo(file);

                    // Skip over files that end in .BAK
                    if (!fi.Name.ToUpper().EndsWith(".BAK"))
                    {
                        // Pull data from the CSV
                        List<AppointmentReminder> reminders = ReadData(file);

                        string destinationFile = file + ".bak";

                        // Queue up the calls to place
                        lock (m_SyncVar)
                        {
                            foreach (AppointmentReminder reminder in reminders)
                            {
                                m_Queue.Enqueue(reminder);
                            }
                        }

                        // Rename the file so that we don't call twice.
                        try
                        {
                            File.Move(file, destinationFile);
                            m_Log.Write("Finished processing file: {0}", file);
                        }
                        catch (Exception ex)
                        {
                            m_Log.WriteException(ex, "Error renaming file!");
                        }
                    }
                }

                // Check to see if the application is still active. If not, we stop execution
                // We check every 5 seconds for new files
                bool isSignalled = AliveEvent.WaitOne(5000);

                if (isSignalled)
                {
                    break;
                }

            }

            m_Log.Write("Ended ReadFiles Thread");
        }

        /// <summary>
        /// This method is responsible for reading data from the CSV file. We've included a simple CSV Parser that reads data into a datatable.
        /// It should be easy to replace this logic with logic that reads data from a database if you would like.
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        private List<AppointmentReminder> ReadData(string filename)
        {
            List<AppointmentReminder> returnValue = new List<AppointmentReminder>();

            using (TextReader tr = new StreamReader(filename))
            {
                DataTable dt = Helper.CsvParser.Parse(m_Log, tr, true);

                if (dt != null)
                {
                    foreach (DataRow row in dt.Rows)
                    {
                        AppointmentReminder reminder = new AppointmentReminder();
                        string sAppointmentTime = row["AppointmentTime"].ToString();
                        DateTime appointmentTime = DateTime.Now;

                        DateTime.TryParse(sAppointmentTime, out appointmentTime);
                        reminder.AppointmentTime = appointmentTime;
                        reminder.CallerId = row["CallerId"].ToString();
                        string[] filesToPlay = row["FilesToPlay"].ToString().Split('|');
                        reminder.FilesToPlay = filesToPlay;

                        reminder.FirstName = row["FirstName"].ToString();
                        reminder.LastName = row["LastName"].ToString();
                        reminder.PhoneNumber = row["PhoneNumber"].ToString();
                        reminder.UniqueId = row["UniqueId"].ToString();

                        returnValue.Add(reminder);
                    }
                }
            }

            return returnValue;
        }

    }
}
