﻿using System;
using System.Collections.Generic;
using System.Text;
using VoiceElements.Client;
using VoiceElements.Common;
using System.Threading;

namespace VoiceApp
{
    public class InboundIvr
    {
        // The Channel Resource of the inbound caller.
        private ChannelResource m_ChannelResource;

        // The Voice Resourceo of the inbound caller
        private VoiceResource m_VoiceResource;

        // The Telephony Server
        private TelephonyServer m_TelephonyServer;

        // The current application log
        private Log Log = IvrApplication.Log;

        // This is the directory where the application is located
        private string WorkingFolder = IvrApplication.WorkingFolder;

        // This reset event will get signalled when either side hangs up. That way the call terminates immediately.
        private ManualResetEvent m_HangupEvent = new ManualResetEvent(false);

        // This channel resource will be used to make the outbound call
        private ChannelResource m_OutboundChannelResource = null;
        

        public InboundIvr(TelephonyServer telephonyServer, ChannelResource channelResource, VoiceResource voiceResource)
        {
            m_TelephonyServer = telephonyServer;
            m_ChannelResource = channelResource;
            m_VoiceResource = voiceResource;
        }


        public void RunScript()
        {
            try
            {
                // Answer the phone call
                m_ChannelResource.Answer();

                // This will get fired when the user hangs up the phone. 
                m_ChannelResource.Disconnected += new Disconnected(m_ChannelResource_Disconnected);

                // This will return if the user hangs up or once 1000 ms has passed it's a safe way of calling thread.wait
                // The reason this is safer is because if someone hangs up immediately after the answer the channel resource will be locked up until that method returns.
                m_HangupEvent.WaitOne(1000);

                string phoneNumber = "";
                bool exitLoop = false;

                // It's a good idea to place a limit on the number of tries that a user can attempt.
                // Otherwise, you can get phone calls that may last a very long time if a user dials in and keeps the line open.
                for (int i = 0; i < 3; i++)
                {
                    // This tells VE to return from subsequent Voice Resource actions if a digit is pressed
                    // If a user presses a digit while TerminationDigits are set to ANY the other plays will not occur.
                    m_VoiceResource.TerminationDigits = "ANY";
                    m_VoiceResource.PlayTTS("Thanks for calling us");
                    m_VoiceResource.PlayTTS("Please press 1 for sales");
                    m_VoiceResource.PlayTTS("Press 2 for support");
                    m_VoiceResource.PlayTTS("Press 3 to lookup an employee using speech");

                    // This tells VE to return if any of the conditions are true: 1 digit is pressed, 10 seconds have passed or ANY digit is pressed
                    m_VoiceResource.GetDigits(1, 10, "ANY");

                    if (m_VoiceResource.DigitBuffer == "1" || m_VoiceResource.DigitBuffer == "2" || m_VoiceResource.DigitBuffer == "3")
                    {
                        exitLoop = true;
                    }

                    if (exitLoop)
                    {
                        break;
                    }
                }

                // Disconnect the user since they've exhausted the number of tries
                if (!exitLoop)
                {
                    m_VoiceResource.PlayTTS("Goodbye");
                    m_ChannelResource.Disconnect();
                    return;
                }

                if (m_VoiceResource.DigitBuffer == "1" || m_VoiceResource.DigitBuffer == "2")
                {
                    // If there aren't resources available you will get back an exception.
                    try
                    {
                        m_OutboundChannelResource = m_TelephonyServer.GetChannel();
                    }
                    catch (Exception ex)
                    {
                        Log.WriteException(ex, "There was an error getting the channel resource!");
                        return;
                    }

                    // Set the maximum dial time. This means we'll try placing a phone call for 45 seconds. 
                    //It will return with a NoAnswer if the phone isn't answered in that time.
                    m_OutboundChannelResource.MaximumTime = 45;

                    // We subscribe to an event that way we know if the called party hangs up.
                    m_OutboundChannelResource.Disconnected += new Disconnected(m_ChannelResource_Disconnected);

                    // We route the resources together. This is so the inbound caller can "hear" the outbound call
                    // When you route before performing a dial it will allow the caller to hear the ringing
                    m_ChannelResource.RouteFull(m_OutboundChannelResource);

                    // This means that the call returns as soon as it gets answered, if the maximum time passes, or if the call failes
                    m_OutboundChannelResource.CallProgress = CallProgress.WaitForConnect;

                    // Now we dial the phone number
                    DialResult dr = m_OutboundChannelResource.Dial(phoneNumber);

                    switch (dr)
                    {
                        case DialResult.Connected:
                        case DialResult.HumanDetected:
                        case DialResult.MachineDetected:
                        case DialResult.Successful:
                            break;
                        default:
                            // If it's not a successfull call we call Set. This means the wait one call below will return immediately allowing the finally block to execute.
                            m_HangupEvent.Set();
                            break;
                    }

                    // This allows the callers to speak until either side hangs up.
                    // We "set" this reset event when a phone call is hungup
                    m_HangupEvent.WaitOne();
                }
                else
                {
                    // Now we'll use speech recognition.

                    try
                    {
                        string gramFile = this.WorkingFolder + "EmployeeList.gram";

                        // We will loop up to 3 times to get an accepted response.
                        for (int i = 0; i < 3; i++)
                        {
                            m_VoiceResource.SpeechRecognitionEnabled = true;
                            m_VoiceResource.SpeechRecognitionGrammarFile = gramFile;
                            m_VoiceResource.SpeechRecognitionMode = VoiceElements.Interface.SpeechRecognitionMode.MultiplePlays;

                            // Set how long we wait until we timeout
                            m_VoiceResource.MaximumTime = 10;

                            // This determines if we allow the user to "Interrupt" the play, by saying something early.
                            m_VoiceResource.SpeechRecognitionPermitBargeIn = true;

                            // Determine how many digits we should accept
                            m_VoiceResource.MaximumDigits = 1;

                            m_VoiceResource.PlayTTS("Please say Yes or No, or press 0 to be transferred to an operator");

                            TerminationCode tc = m_VoiceResource.GetResponse();

                            // We should disable Speech Recognition now, otherwise it could be inadvertantly used on subsequent plays
                            m_VoiceResource.SpeechRecognitionEnabled = false;

                            Log.Write("TC: {0} DigitBuffer: {1} Score: {2} Word: {3}", tc, m_VoiceResource.DigitBuffer, m_VoiceResource.SpeechRecognitionScore, m_VoiceResource.SpeechRecognitionReturnedWord);
                            if (tc == TerminationCode.Speech)
                            {
                                if (m_VoiceResource.SpeechRecognitionScore >= 700) // Scores range between 0 and 1000. Generally, anything above 700 could be considered a strong positive detection
                                {
                                    string word = m_VoiceResource.SpeechRecognitionReturnedWord;
                                    m_VoiceResource.PlayTTS("You said: " + word + ". Goodbye");
                                    break;
                                }
                            }
                            else if (m_VoiceResource.TerminationCodeFlag(TerminationCode.Digit) || m_VoiceResource.TerminationCodeFlag(TerminationCode.MaximumDTMF))
                            {
                                if (m_VoiceResource.DigitBuffer == "0")
                                {
                                    // Transfer to operator
                                    break;
                                }
                            }

                            m_VoiceResource.PlayTTS("I'm sorry, I did not understand your response");
                        }
                    }
                    catch (HangupException hex)
                    {
                        Log.Write("Caller Hungup!");
                    }
                    catch (Exception ex)
                    {
                        Log.WriteException(ex, "Unexpected exception");
                    }
                }
            }
            catch (HangupException hex)
            {
                // When we are performing a voice operation, we throw a hangup exception if the user hangsup before or during the voice function.
                Log.Write("Hangup!");
            }
            catch (Exception ex)
            {
                Log.WriteException(ex, "InboundIvr::RunScript() - Unexpected Exception!");
            }
            finally
            {
                // we only need to dispose any thing that we've created in this method

                try
                {
                    m_ChannelResource.Disconnected -= new Disconnected(m_ChannelResource_Disconnected);
                }
                catch { }

                if (m_OutboundChannelResource != null)
                {
                    // Disconnect the call
                    try
                    {
                        m_OutboundChannelResource.Disconnect();
                    }
                    catch { }

                    // Unsubscribe the event
                    try
                    {
                        m_OutboundChannelResource.Disconnected -= new Disconnected(m_ChannelResource_Disconnected);
                    }
                    catch { }

                    // Dispose the channel resource
                    try
                    {
                        m_OutboundChannelResource.Dispose();
                    }
                    catch { }

                    m_OutboundChannelResource = null;
                }

            }
        }

        void m_ChannelResource_Disconnected(object sender, DisconnectedEventArgs e)
        {
            Log.Write("Customer hungup!");
            m_HangupEvent.Set(); // This notifies
        }
    }
}
