using System;
using System.Collections.Generic;
using System.Text;
using VoiceElements.Common;
using VoiceElements.Client;
using System.Threading;
using System.IO;

namespace VoiceApp
{
    public class IvrApplication
    {
        private static object s_SyncVar = new object();

        public static object SyncVar
        {
            get { return IvrApplication.s_SyncVar; }
        }

        private static Log s_Log;
        public static Log Log
        {
            get { return s_Log; }
        }

        private static State s_State;

        public static State State
        {
            get { return s_State; }
        }

        private static Thread s_MainCodeThread;

        public static Thread MainCodeThread
        {
            get { return s_MainCodeThread; }
        }

        private static AutoResetEvent s_ThreadEvent = new AutoResetEvent(false);

        public static AutoResetEvent ThreadEvent
        {
            get { return s_ThreadEvent; }
        }

        /// <summary>
        /// We set the working folder to a value in the configuration file.
        /// </summary>
        private static string s_WorkingFolder = "";

        public static string WorkingFolder
        {
            get { return s_WorkingFolder; }
        }

        static IvrApplication()
        {
            // Constructor
            s_Log = new Log("IvrApplication.Log");
            Log.Write("IvrApplication Constructor Complete");
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
        }

        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            if (e.ExceptionObject is Exception)
            {
                Log.WriteException((Exception)e.ExceptionObject, "Domain Level Unhandled Exception");
            }
            else
            {
                Log.Write("Domain Level Unhandled Exception - No Exception Object");
            }
        }

        public static void Start()
        {

            lock (SyncVar)
            {
                if (State == State.Stopped)
                {
                    s_State = State.Starting;
                    ThreadStart ts = new ThreadStart(MainCode);
                    s_MainCodeThread = new Thread(ts);
                    s_MainCodeThread.Name = "IvrApplication";
                    s_MainCodeThread.Start();
                    Log.Write("IvrApplication Starting...");
                }
                else
                {
                    Log.Write("IvrApplication is in the " + State.ToString() + " state.  Cannot start IvrApplication at this time.");
                }
            }
        }

        public static void StopImmediate()
        {
            lock (SyncVar)
            {
                if (State == State.Running || State == State.StoppingControlled)
                {
                    s_State = State.StoppingImmediate;
                    ThreadEvent.Set();
                    Log.Write("IvrApplication StoppingImmediate.");
                }
                else
                {
                    Log.Write("IvrApplication is in the " + State.ToString() + " state.  Cannot stop IvrApplication at this time.");
                }
            }
        }

        public static void StopControlled()
        {
            lock (SyncVar)
            {
                if (State == State.Running)
                {
                    s_State = State.StoppingControlled;
                    ThreadEvent.Set();
                    Log.Write("IvrApplication StoppingControlled.");
                }
                else
                {
                    Log.Write("IvrApplication is in the " + State.ToString() + " state.  Cannot stop IvrApplication at this time.");
                }
            }
        }

        public static TelephonyServer s_TelephonyServer = null;

        /// <summary>
        /// The dial manager contains the logic for reading the CSV files and starting up calls.
        /// There are two threads that we need to start in the "MainCode" method below.
        /// We stop the execution of both of those threads in the finally block of the MainCode method.
        /// </summary>
        public static DialManager s_DialManager = null;

        public static void MainCode()
        {

            try
            {
                s_WorkingFolder = Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location);

                Log.Write("IvrApplication::MainCode() Starting...");

                // Start Other Threads...
                try
                {

                    // UPDATE YOUR SERVER ADDRESS HERE
                    string sIpaddress = @"gtcp://" + Properties.Settings.Default.TelephonyServer + ":54331";

                    Log.Write("Connecting to: {0}", sIpaddress);

                    // CHANGE YOUR USERNAME AND PASSWORD HERE
                    s_TelephonyServer = new TelephonyServer(sIpaddress, Properties.Settings.Default.Username, Properties.Settings.Default.Password);

                    // CHANGE YOUR CACHE MODE HERE
                    //
                    // Client Session mode means that the server will stream and cache the files to/from your client machine.
                    // Files are flushed after you disconnect.
                    //
                    // Server mode means that the files reside on the server and will use the full path name to find them there.
                    // Server mode can only be used on your own dedicate VE server.

                    s_TelephonyServer.CacheMode = VoiceElements.Interface.CacheMode.ClientSession;


                    // This is a strictly outbound application so we comment this out.
                    // SUBSCRIBE to the new call event.
                    //s_TelephonyServer.NewCall += new VoiceElements.Client.NewCall(s_TelephonyServer_NewCall);
                    //s_TelephonyServer.RegisterDNIS();

                    // Subscribe to the connection events to allow you to reconnect if something happens to the internet connection.
                    // If you are running your own VE server, this is less likely to happen except when you restart your VE server.
                    s_TelephonyServer.ConnectionLost += new ConnectionLost(s_TelephonyServer_ConnectionLost);
                    s_TelephonyServer.ConnectionRestored += new ConnectionRestored(s_TelephonyServer_ConnectionRestored);

                    // Now we need to create a new instance of 
                    s_DialManager = new DialManager();

                    // Now that everything is setup. We can setup our different threads
                    System.Threading.Thread threadPlaceCalls = new Thread(s_DialManager.PlaceCalls);
                    threadPlaceCalls.Name = "PlaceCalls";

                    System.Threading.Thread threadReadFiles = new Thread(s_DialManager.ReadFiles);
                    threadReadFiles.Name = "ReadFiles";

                    // Now we start up the different rheads
                    threadPlaceCalls.Start();
                    threadReadFiles.Start();
                }
                catch (Exception ex)
                {
                    try
                    {
                        if (s_TelephonyServer != null)
                        {
                            s_TelephonyServer.Dispose();
                        }
                    }
                    catch (Exception) { }

                    Log.Write("IvrApplication::MainCode() Exception: " + ex.Message + "\r\n" + ex.StackTrace);
                    throw ex;
                }

                Log.Write("VoiceElementsClient Version: {0}", s_TelephonyServer.GetClientVersion());
                Log.Write("VoiceElementsServer Version: {0}", s_TelephonyServer.GetServerVersion());

                lock (SyncVar)
                {
                    s_State = State.Running;
                }

                Log.Write("IvrApplication::MainCode() Running...");

                while (true)
                {

                    // Waits for some asyncronous event.
                    ThreadEvent.WaitOne(10000, false);

                    // At this point you are in control.  You can farm out calls from a database, 
                    // or you could code the IvrInteractive Form and create a GUI for handling you calls.
                    // Follow the example from the Sampler on how to make an outbound class for new calls.

                    lock (SyncVar)
                    {
                        if (State != State.Running) break;
                    }
                }

                s_TelephonyServer.Dispose();
                s_TelephonyServer = null;

                // Must be shutting down...

                if (State == State.StoppingControlled)
                {
                    Log.Write("IvrApplication::MainCode() StoppingControlled...");
                }

                if (State == State.StoppingImmediate)
                {
                    Log.Write("IvrApplication::MainCode() StoppingImmediate...");
                }

                lock (SyncVar)
                {
                    s_State = State.Stopped;
                    Log.Write("IvrApplication::MainCode() Stopped.");
                }

            }
            catch (Exception ex)
            {
                Log.Write("IvrApplication::MainCode() Exception" + ex.Message + "\r\n" + ex.StackTrace);
                s_State = State.Stopped;
            }
            finally
            {
                s_MainCodeThread = null;
            }
        }

        static void s_TelephonyServer_ConnectionRestored(object sender, ConnectionRestoredEventArgs e)
        {

            // When the connection is restored you must reset your cache mode and re-register the DNIS.

            s_TelephonyServer.CacheMode = VoiceElements.Interface.CacheMode.ClientSession;
            //s_TelephonyServer.CacheMode = VoiceElements.Interface.CacheMode.Server;
            s_TelephonyServer.RegisterDNIS();

            Log.Write("The Connection to the server was successfully restored!");

        }

        static void s_TelephonyServer_ConnectionLost(object sender, ConnectionLostEventArgs e)
        {

            // You could also send an email to yourself to let you know that the server was down.
            Log.Write("The Connection to the server was lost.");
        }

        static void ChannelResource_Disconnected(object sender, DisconnectedEventArgs e)
        {
            Log.Write("Disconnected (Meaning Caller Hung-up) Event Recevied.");
        }
    }
}
