﻿using System;
using System.Collections.Generic;
using System.Text;
using VoiceElements.Common;
using VoiceElements.Client;
using System.Threading;

namespace VoiceApp
{
    public class OutboundDial
    {
        public AppointmentReminder CallData
        {
            get;
            set;
        }

        private ChannelResource m_ChannelResource;

        private VoiceResource m_VoiceResource;

        private TelephonyServer m_TelephonyServer;

        private Log m_Log = IvrApplication.Log;

        private string m_WorkingFolder = IvrApplication.WorkingFolder;

        private AppointmentReminderResult m_Result = new AppointmentReminderResult();

        public DialManager m_DialManager
        {
            get;
            set;
        }

        public OutboundDial (TelephonyServer telephonyServer, DialManager dialManager)
        {
            m_TelephonyServer = telephonyServer;
            m_DialManager = dialManager;
        }

        public bool PlayFiles(bool useBeepDetector)
        {
            m_Log.WriteWithId(CallData.UniqueId, "Started play files!");
            bool isBeepDetected = false;
            TerminationCode tc = TerminationCode.Normal;

            // Set this so that we don't allow users to terminate the play
            m_VoiceResource.TerminationDigits = "";
            
            if(useBeepDetector)
            {
                m_VoiceResource.BeepDetectionStart();
            }
            foreach (string filename in CallData.FilesToPlay)
            {
                switch (filename.ToUpper().Trim())
                {
                    case "FIRSTNAME":
                        tc = m_VoiceResource.PlayTTS(CallData.FirstName);
                        break;
                    case "LASTNAME":
                        tc = m_VoiceResource.PlayTTS(CallData.LastName);
                        break;
                    case "APPOINTMENTTIME":
                        tc = m_VoiceResource.PlayTTS(CallData.AppointmentTime.ToShortTimeString());
                        break;
                    default:
                        tc = m_VoiceResource.Play(m_WorkingFolder + @"\" + filename);
                        // play the filename
                        break;
                }

                if (tc == TerminationCode.Beep)
                {
                    isBeepDetected = true;
                    break;
                }
            }

            if(useBeepDetector)
            {
                m_Log.WriteWithId(CallData.UniqueId, "Detected a beep!");
                // Now we stop the beep detector
                m_VoiceResource.BeepDetectionStop();
            }

            m_Log.WriteWithId(CallData.UniqueId, "Ended Play Files");
            return isBeepDetected;
        }

        public void RunScript()
        {
            try
            {
                try
                {
                    m_ChannelResource = m_TelephonyServer.GetChannel();
                    m_VoiceResource = m_ChannelResource.VoiceResource;
                }
                catch(Exception ex)
                {
                    m_Log.Write("Could not get Channel");
                    return;
                }

                m_Log.WriteWithId(CallData.UniqueId, "Got resource. Setting dial parameters");

                // The originating phone number sets the number that gets displayed 
                m_ChannelResource.OriginatingPhoneNumber = CallData.CallerId;

                // This means that it will return as soon as the call is connected
                // Alternatively you can use a call progress mode of AnalyzeCall to try to detect if it's a human or machine
                m_ChannelResource.CallProgress = CallProgress.WaitForConnect;
                
                // Set the call start time
                m_Result.CallStart = DateTime.Now;

                // Dial out
                DialResult dr = m_ChannelResource.Dial(CallData.PhoneNumber);
                m_Log.WriteWithId(CallData.UniqueId, "Returned from dial with result: {0}", dr);

                if (dr == DialResult.Connected)
                {
                    // We set if the call is connected or not
                    m_Result.IsCallConnected = true;

                    m_Result.CallConnect = DateTime.Now;

                    // We'll play back all files with the beep detector
                    bool isBeepDetected = PlayFiles(true);

                    // Set the result
                    m_Result.IsBeepDetected = isBeepDetected; 
                    
                    // If we detect a beep, we'll want to replay the files so that the whole message gets played back.
                    if(isBeepDetected)
                    {
                        PlayFiles(false);
                    }
                }
            }
            catch (Exception ex)
            {
                m_Log.WriteException(ex, "OutboundDial::RunScript Exception!");
            }
            finally
            {
                m_Result.CallDisconnect = DateTime.Now;

                if (m_ChannelResource != null)
                {
                    try
                    {
                        m_ChannelResource.Disconnect();
                    }
                    catch { }

                    try
                    {
                        m_ChannelResource.Dispose();
                    }
                    catch { }
                }

                // Signal back that the call is complete. This will ALWAYS get executed
                m_DialManager.DecrementCallCounters();

                // Set the channel resource to null
                m_ChannelResource = null;

                // It's a good practice to update your database in the finally block. That way you don't delay the cleanup of resources
                // You would put your database logic to record call results here.
                // ProcessResult(m_Result);
            }
        }
    }
}
